"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var sids_1 = require("flex-dev-utils/dist/sids");
var random_1 = require("flex-dev-utils/dist/random");
var baseClient_1 = __importDefault(require("./baseClient"));
var services_1 = __importDefault(require("./services"));
var paths_1 = __importDefault(require("../utils/paths"));
var EnvironmentClient = /** @class */ (function (_super) {
    __extends(EnvironmentClient, _super);
    function EnvironmentClient(auth, serviceSid) {
        var _this = _super.call(this, auth, services_1.default.getBaseUrl() + "/Services/" + serviceSid) || this;
        /**
         * Returns the {@link Environment} that has the same name as the packageName
         *
         * @param create  if set to true, will create an environment if not found
         */
        _this.get = function (create) {
            if (create === void 0) { create = true; }
            return _this.list()
                .then(function (resource) { return resource.environments.find(function (s) { return s.unique_name === paths_1.default.packageName; }); })
                .then(function (environment) {
                if (!environment) {
                    if (create) {
                        return _this.create();
                    }
                    else {
                        throw new Error("No environment with unique_name " + paths_1.default.packageName + " was found");
                    }
                }
                return environment;
            });
        };
        /**
         * Creates an environment with the package name
         */
        _this.create = function () {
            return _this.list()
                .then(function (resource) { return resource.environments; })
                .then(function (environments) {
                var list = environments.map(function (environment) { return environment.domain_suffix; });
                return random_1.randomString(EnvironmentClient.DomainSuffixLength, list);
            })
                .then(function (domainSuffix) {
                return _this.http
                    .post(EnvironmentClient.BaseUri, {
                    UniqueName: paths_1.default.packageName,
                    DomainSuffix: domainSuffix,
                });
            });
        };
        /**
         * Removes the Environment
         */
        _this.remove = function (sid) {
            if (!sids_1.isSidOfType(sid, sids_1.SidPrefix.EnvironmentSid)) {
                throw new Error(sid + " is not of type " + sids_1.SidPrefix.EnvironmentSid);
            }
            return _this.http
                .delete(EnvironmentClient.BaseUri + "/" + sid);
        };
        /**
         * Fetches a list of {@link Environment}
         */
        _this.list = function () {
            return _this.http
                .get(EnvironmentClient.BaseUri);
        };
        if (!sids_1.isSidOfType(serviceSid, sids_1.SidPrefix.ServiceSid)) {
            throw new Error("ServiceSid " + serviceSid + " is not valid");
        }
        return _this;
    }
    EnvironmentClient.BaseUri = 'Environments';
    EnvironmentClient.DomainSuffixLength = 5;
    return EnvironmentClient;
}(baseClient_1.default));
exports.default = EnvironmentClient;
//# sourceMappingURL=environments.js.map