"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports._checkPluginCount = exports._readIndexPage = exports._verifyPackageVersion = exports._checkExternalDepsVersions = exports._checkPublicDirSync = exports._checkCracoConfig = exports._checkAppConfig = void 0;
var flex_dev_utils_1 = require("flex-dev-utils");
var fs_1 = require("flex-dev-utils/dist/fs");
var fs_2 = require("fs");
var path_1 = require("path");
var prints_1 = require("../prints");
var require_1 = require("../utils/require");
var run_1 = __importDefault(require("../utils/run"));
var flexUIPkgPath = path_1.join(require_1.nodeModulesPath, '@twilio/flex-ui/package.json');
var srcIndexPath = path_1.join(process.cwd(), 'src', 'index');
var appConfigPath = path_1.join(process.cwd(), 'public', 'appConfig.js');
var cracoConfigPath = path_1.join(process.cwd(), 'craco.config.js');
var indexSourcePath = path_1.join(__dirname, '..', '..', 'dev_assets', 'index.html');
var indexTargetPath = path_1.join(process.cwd(), 'public', 'index.html');
var extensions = ['js', 'jsx', 'ts', 'tsx'];
var PackagesToVerify = [
    'react',
    'react-dom',
];
/**
 * Checks appConfig exists
 *
 * @private
 */
exports._checkAppConfig = function () {
    if (!fs_2.existsSync(appConfigPath)) {
        prints_1.appConfigMissing();
        return process.exit(1);
    }
};
/**
 * Checks that craco-config path exists
 * @private
 */
exports._checkCracoConfig = function () {
    if (!fs_2.existsSync(cracoConfigPath)) {
        prints_1.cracoConfigMissing();
        return process.exit(1);
    }
};
/**
 * Syncs required files in public/ dir
 *
 * @param allowSkip whether to allow skip
 * @private
 */
exports._checkPublicDirSync = function (allowSkip) {
    try {
        fs_2.copyFileSync(indexSourcePath, indexTargetPath);
    }
    catch (e) {
        prints_1.publicDirCopyFailed(e, allowSkip);
        return process.exit(1);
    }
};
/**
 * Checks the version of external libraries and exists if customer is using another version
 *
 * allowSkip  whether to allow skip
 * @private
 */
/* istanbul ignore next */
exports._checkExternalDepsVersions = function (allowSkip, allowReact) {
    var flexUIPkg = require(flexUIPkgPath);
    PackagesToVerify.forEach(function (name) { return exports._verifyPackageVersion(flexUIPkg, allowSkip, allowReact, name); });
};
/**
 * Checks the version of external libraries and exists if customer is using another version
 *
 * @param flexUIPkg   the flex-ui package.json
 * @param allowSkip   whether to allow skip
 * @param allowReact  whether to allow unbundled react
 * @param name        the package to check
 * @private
 */
exports._verifyPackageVersion = function (flexUIPkg, allowSkip, allowReact, name) {
    var expectedDependency = flexUIPkg.dependencies[name];
    var supportsUnbundled = flex_dev_utils_1.semver.satisfies(flexUIPkg.version, '>=1.19.0');
    if (!expectedDependency) {
        prints_1.expectedDependencyNotFound(name);
        return process.exit(1);
    }
    // @ts-ignore
    var requiredVersion = flex_dev_utils_1.semver.coerce(expectedDependency).version;
    var installedVersion = require_1.getPackageVersion(name);
    if (requiredVersion !== installedVersion) {
        if (allowReact) {
            if (supportsUnbundled) {
                return;
            }
            prints_1.unbundledReactMismatch(flexUIPkg.version, name, installedVersion, allowSkip);
        }
        else {
            prints_1.versionMismatch(name, installedVersion, requiredVersion, allowSkip);
        }
        if (!allowSkip) {
            return process.exit(1);
        }
    }
};
/**
 * Returns the content of src/index
 * @private
 */
/* istanbul ignore next */
exports._readIndexPage = function () {
    var match = extensions
        .map(function (ext) { return srcIndexPath + "." + ext; })
        .find(function (file) { return fs_1.checkFilesExist(file); });
    if (match) {
        return fs_2.readFileSync(match, 'utf8');
    }
    throw new flex_dev_utils_1.FlexPluginError('No index file was found in your src directory');
};
/**
 * Checks how many plugins this single JS bundle is exporting
 * You can only have one plugin per JS bundle
 * @private
 */
exports._checkPluginCount = function () {
    var content = exports._readIndexPage();
    var match = content.match(/loadPlugin/g);
    if (!match || match.length === 0) {
        prints_1.loadPluginCountError(0);
        return process.exit(1);
    }
    if (match.length > 1) {
        prints_1.loadPluginCountError(match.length);
        return process.exit(1);
    }
};
/**
 * Runs pre-start/build checks
 */
var checkStart = function () { return __awaiter(void 0, void 0, void 0, function () {
    var allowSkip, allowReact;
    return __generator(this, function (_a) {
        flex_dev_utils_1.logger.debug('Checking Flex plugin project directory');
        allowSkip = process.env.SKIP_PREFLIGHT_CHECK === 'true';
        allowReact = process.env.UNBUNDLED_REACT === 'true';
        exports._checkAppConfig();
        exports._checkCracoConfig();
        exports._checkPublicDirSync(allowSkip);
        exports._checkExternalDepsVersions(allowSkip, allowReact);
        exports._checkPluginCount();
        return [2 /*return*/];
    });
}); };
run_1.default(checkStart);
exports.default = checkStart;
//# sourceMappingURL=check-start.js.map