import { IncomingMessage, ServerResponse } from 'http';
export interface Plugin {
    src: string;
    name: string;
    enabled: boolean;
    remote?: boolean;
}
/**
 * Returns local plugins from  public/plugins.json
 * @private
 */
export declare const _getLocalPlugins: () => Plugin[];
/**
 * Generates the response headers
 *
 * @param port  the port the browser will be running on
 * @private
 */
export declare const _getHeaders: (port: number) => {
    'Access-Control-Allow-Origin': string;
    'Access-Control-Allow-Methods': string;
    'Access-Control-Allow-Headers': string;
    'Access-Control-Allow-Credentials': string;
    'Content-Type': string;
};
/**
 * Fetches the Plugins from Flex
 *
 * @param token     the JWE Token
 * @param version   the Flex version
 */
export declare const _getRemotePlugins: (token: string, version: string) => Promise<Plugin[]>;
/**
 * Rebase plugins with local plugins
 * @param remotePlugins   the plugins returned from Flex
 * @private
 */
export declare const _rebasePlugins: (remotePlugins: Plugin[]) => (Plugin | null | undefined)[];
/**
 * Basic server to fetch plugins from Flex and return to the local dev-server
 * @param browserPort  the port of browser
 * @private
 */
export declare const _server: (browserPort: number) => (req: IncomingMessage, res: ServerResponse) => Promise<void>;
/**
 * Generates the pluginsService.js
 *
 * @param port  the port the local dev-server is running
 * @private
 */
export declare const _generatePluginServiceConfig: (port: number) => void;
/**
 * Runs a local dev-server to proxy requests to Flex plugin service
 * @param browserPort   the port the browser is running
 */
declare const pluginServer: (browserPort: string | number) => Promise<void>;
export default pluginServer;
